<?php

namespace Evoluted\Deployer\VhostApi;

use function Deployer\has;
use function Deployer\get;
use function Deployer\writeln;

/**
 * Notify the Vhost API about a (possibly new) hostname
 *
 * @param string $hostname
 * @param string $applicationType E.g. "laravel", "cake"
 * @param string $phpVersion E.g. "7.1"
 * @param string $serverType Not currently used
 * @return bool Whether the request to the vhost API was successful
 */
function notifyVhostApiManager($hostname, $applicationType, $phpVersion, $serverType) {

    // If it is UAT
    if (has('vhost_api_host')) {
        $vhostApiManagerHost = get('vhost_api_host');
    } elseif (strtolower(getenv('CI_BUILD_REF_SLUG')) == 'uat' || strtolower(getenv('IS_UAT') == 'yes')) {
        $vhostApiManagerHost = 'https://vhost-api.uat.evoluted.net';
    } else {
        $vhostApiManagerHost = 'http://vhost-api.cd.evoluted.net';
    }

    if (has('application/public_path')) {
        $applicationPublicPath = get('application/public_path');
    } else {
        $applicationPublicPath = getApplicationPublicPath($applicationType);
    }

    $bodyArray = [
        'template' => 'nginx',
        'server_name' => $hostname,
        'root' => get('release_path') . DIRECTORY_SEPARATOR . $applicationPublicPath,
        'server_type' => 'php-fpm',
        'php_version' => $phpVersion
    ];

    // Merge in any custom params added by the applications deployer config.
    if (has('vhost_parameters')) {
        $bodyArray = array_merge($bodyArray, get('vhost_parameters'));
    }

    $curl = curl_init();

    writeln('Sending deploy request to ' . $hostname);

    curl_setopt_array($curl, array(
        CURLOPT_URL => $vhostApiManagerHost . "/api/vhost/create",
        CURLOPT_RETURNTRANSFER => true,
        CURLOPT_ENCODING => "",
        CURLOPT_MAXREDIRS => 10,
        CURLOPT_TIMEOUT => 30,
        CURLOPT_HTTP_VERSION => CURL_HTTP_VERSION_1_1,
        CURLOPT_CUSTOMREQUEST => "POST",
        CURLOPT_POSTFIELDS => json_encode($bodyArray),
        CURLOPT_HTTPHEADER => array(
            "Accept: application/json",
            "Authorization: Bearer " . getVhostManagerApiAccessToken(),
            "Content-Type: application/json",
        ),
    ));

    $response = curl_exec($curl);
    $err = curl_error($curl);

    curl_close($curl);

    if ($err) {
        echo "cURL Error #:" . $err;
    } else {
        var_dump($response);
        $jsonResponse = json_decode($response, true);

        if (!empty($jsonResponse['success']) && $jsonResponse['success'] == true) {
            return true;
        } else {
            return false;
        }
    }
}

/**
 * Returns the API manager access token, uses one set in deploy.php if it is with the key vhostmanagerapi/access_token
 * otherwise uses default one
 * @return string
 */
function getVhostManagerApiAccessToken() {
    if (has('vhostmanagerapi/access_token')) {
        return get('vhostmanagerapi/access_token');
    }
    return 'eyJ0eXAiOiJKV1QiLCJhbGciOiJSUzI1NiIsImp0aSI6Ijg3YjNlN2NjYzU0YmQwYzI0MThhZTlhZjVhNDFkOTQ2OThiNGE2ZWE5OWYxMTM1NmE0NzgxNGNiNjEyYTRhYmZlOTliM2Q2ZWFkNTEwNTYxIn0.eyJhdWQiOiIxIiwianRpIjoiODdiM2U3Y2NjNTRiZDBjMjQxOGFlOWFmNWE0MWQ5NDY5OGI0YTZlYTk5ZjExMzU2YTQ3ODE0Y2I2MTJhNGFiZmU5OWIzZDZlYWQ1MTA1NjEiLCJpYXQiOjE1OTQwNDk5MTYsIm5iZiI6MTU5NDA0OTkxNiwiZXhwIjoxNjI1NTg1OTE2LCJzdWIiOiI2Iiwic2NvcGVzIjpbXX0.cZiuTqv2Oozu4vwjNUeMXVFb99rqzAa8IQFIAr3HhW9CH_0UZC9NL_iPOOHc4AH8X-eyizFyGJwfQKcO1ln-Ri1Slfnur6NGPM72av73j8HcIciFZapc3lDgkVytiOhpQDROsRB-xiZarxXdlDWvox-J2qolDq0kdKg7bKXqOb2w7VbOsJXA8F_iOIgCZoCbK7oDhHjt_hEe_7pwn1-pUMOb_pzm1Tr3c842ohdyFpCNkx-15RI5u77BHNMYCxrFBC48ekNBVTIv8h5fPw43TCanKmjqYKanrUJg_jrRR88PfHyywqGZ6fDXdbJPE5c9NYlCP7FFo7XqoB7EtdTMehA3a3xoDDKOmn4R4Vu24vQl7zlrnlYBPsj5S2HVC9NQLi1X-XId6-2vR48R7gLoDmtgP-IcdmrBbqyHjKLZGnDkePsUiTwFtUbkf57efQ9l_2NTaOFXhTKOtauH3XwZmJvGfd7BB81gjZJJpZrdvigVCJviWY1-pCZMDHiTURc2kvzFWpHLskUnTm22_2rRAYKn_eTYUeM4byl3dNS7bpPUZ09Pcy4VTmlyfaMT0NUyGzQhTVBLAbWhkMz8kEi4ATb2OBOHzhV5QrXWKXJq73dOwqfx45J73OajzAe-McRMPvvLY7wDDKUGPjxDMjhu7oJ8orrX6teYTlwqd5B387Y';
}

/**
 * Gets the public directory path for a given application type
 *
 * @param $applicationType
 * @return string The correct relative path to the application public directory e.g. for Cakephp2 app/webroot, Laravel
 *         public
 */
function getApplicationPublicPath($applicationType) {
    switch ($applicationType) {
        case 'cake':
        case 'cakephp':
        case 'cakephp2':
            return 'app/webroot';
        case 'laravel':
        default:
            return 'public';
    }
}
